searchNodes=[{"ref":"Mix.Tasks.ZigDoc.html","title":"Mix.Tasks.ZigDoc","type":"task","doc":"Used to generate documentation from your zig code and incorporate it into your Elixir documentation. Generally speaking, zig structs will be documented as Elixir modules and zig functions will be incorporated as Elixir functions. If you want to make the zig docgen process override the Elixir mix docs task, add this to your mix.exs: def project do [ ... aliases: [docs: &quot;zig_doc&quot;], ... ] end"},{"ref":"Zigler.html","title":"Zigler","type":"module","doc":"Inline NIF support for Zig Motivation Zig is a general-purpose programming language designed for robustness, optimality, and maintainability. The programming philosophy of Zig matches up nicely with the programming philosophy of the BEAM VM and in particular its emphasis on simplicity and structure should very appealing to the practitioners of Elixir. The following features make Zig extremely amenable to inline language support in a BEAM language: simplicity. Zig&#39;s syntax is definable in a simple YACC document and Zig takes a stance against making its featureset more complex (though it may evolve somewhat en route to 1.0) Composability. Zig is unopinionated about how to go about memory allocations. Its allocator interface is very easily able to be backed by the BEAM&#39;s, which means that you have access to generic memory allocation strategies through its composable allocator scheme. C integration. It&#39;s very easy to design C-interop between Zig and C. In fact, Zig is likely to be an easier glue language for C ABIs than C. Basic usage In the BEAM, you can define a NIF by consulting the following document and implementing the appropriate shared object/DLL callbacks. However, Zigler will take care of all of this for you. Simply use Zigler in your module, providing the app atom in the property list. Then, use the sigil_Z/2 macro and write zig code. Any nifs you define should be preceded with the /// nif: function_name/arity zig docstring. Example defmodule MyModule do use Zigler ~Z&quot;&quot;&quot; /// nif: my_func/1 fn my_func(val: i64) i64 { return val + 1; } &quot;&quot;&quot; end Zigler will automatically fill out the appropriate NIF C template, compile the shared object, and bind it into the module pre-compilation. In the case of the example, there will be a MyModule.my_func/1 function call found in the module. Zigler will also make sure that your statically-typed Zig data are guarded when you marshal it from the dynamically-typed BEAM world. However, you may only pass in and return certain types. As an escape hatch, you may use the beam.term type which is equivalent to the ERLNIFTERM type. See erl_nif. Nerves Support Nerves is supported out of the box, and the system should cross-compile to arm ABI as necessary depening on what your nerves :target is. You may also directly specify a zig target using the use Zigler, target: &lt;target&gt; option. Environment Sometimes, you will need to pass the BEAM environment (which is the code execution context, including process info, etc.) into the NIF function. In this case, you should pass it as the first argument, as a beam.env type value. Example defmodule MyModule do use Zigler ~Z&quot;&quot;&quot; /// nif: my_func_with_env/1 fn my_func_with_env(env: beam.env, pid: beam.pid) void { var sendable_term: []u64 = &quot;ping&quot;[0..]; var msg = beam.make_slice(env, sendable_term); var res = e.enif_send(env, pid, env, msg); } &quot;&quot;&quot; end External Libraries If you need to bind static (*.a) or dynamic (*.so) libraries into your module, you may link them with the :libs argument. Note that zig statically binds shared libraries into the assets it creates. This simplifies deployment for you. Example defmodule Blas do use Zigler, libs: [&quot;/usr/lib/x86_64-linux-gnu/blas/libblas.so&quot;], include: [&quot;/usr/include/x86_64-linux-gnu&quot;] ~Z&quot;&quot;&quot; const blas = @cImport({ @cInclude(&quot;cblas.h&quot;); ... Compilation assistance If something should go wrong, Zigler will translate the Zig compiler error into an Elixir compiler error, and let you know exactly which line in the ~Z block it came from. Syntactic Sugar Some of the erlang nif terms can get unwieldy, especially in Zig, which prefers terseness. Each of the basic BEAM types is shadowed by a Zig type in the beam module. The beam struct is always imported into the header of the zig file used, so all zig code in the same directory as the module should have access to the beam struct if they @import(&quot;beam.zig&quot;) Importing files If you need to write code outside of the basic module (you will, for anything non-trivial), just place it in the same directory as your module. Example ~Z&quot;&quot;&quot; const extra_code = @import(&quot;extra_code.zig&quot;); /// nif: use_extra_code/1 fn use_extra_code(val: i64) i64 { return extra_code.extra_fn(val); } &quot;&quot;&quot; If you would like to include a custom c header file, create an include/ directory inside your path tree and it will be available to zig as a default search path as follows: ~Z&quot;&quot;&quot; const c = @cImport({ @cInclude(&quot;my_c_header.h&quot;); }); // nif: my_nif/1 ... &quot;&quot;&quot; Documentation Use the builtin zig /// docstring to write your documentation. If it&#39;s in front of the nif declaration, it will wind up in the correct place in your elixir documentation. See Zigler.Doc for more information on how to document in zig and what to document. See Mix.Tasks.ZigDoc for information on how to get your Elixir project to incorporate zig documentation. Tests Use the builtin zig test keyword to write your internal zig unit tests. These can be imported into an ExUnit module by following this example: defmodule MyTest do use ExUnit.Case use Zigler.Unit zigtest ModuleWithZigCode end See Zigler.Unit for more information."},{"ref":"Zigler.html#sigil_Z/2","title":"Zigler.sigil_Z/2","type":"macro","doc":"declares a string block to be included in the module&#39;s .zig source file. At least one of these blocks must define a nif."},{"ref":"Zigler.html#sigil_z/2","title":"Zigler.sigil_z/2","type":"macro","doc":"like sigil_Z/2, but lets you interpolate values from the outside elixir context using string interpolation (the \#{value} form)"},{"ref":"Zigler.Doc.html","title":"Zigler.Doc","type":"module","doc":"Translates the docstrings from your Zig code into Elixir documentation. For instructions on how to incorporate this into an Elixir project, consult Mix.Tasks.ZigDoc Documentation forms Currently, Zigler recognizes four types of code segments which should be documented. functions types values errors Functions functions have the following signature: pub fn &lt;identifier&gt;(&lt;arguments&gt;) &lt;type&gt; { and may have the property of being comptime which is due to either the function being itself a comptime function or it having a comptime argument. NB only pub functions are documented in Zigler, following the Elixir philosophy that only public functions should be documented. Types types have the following signature: pub const &lt;identifier&gt;=&lt;value&gt;; Values values can have one of the following signatures: pub const &lt;identifier&gt;=&lt;value&gt;; pub var &lt;identifier&gt;=&lt;value&gt;; Note that the constant value form is indistinguishable from the type form without doing a full parse and evaluation of the Zig code. In order to avoid doing this, to disambiguate between the two, you must prepend constant value docststrings with a !value token. Example /// !value /// /// a constant representing the value 47. pub const fortyseven = 47; Errors errors appear inside special error struct and should be documented at the per-value level: Example /// docstring for this error struct (if desired) pub const my_error = error { /// docstring for ErrorEnum1 ErrorEnum1, /// docstring for ErrorEnum2 ErrorEnum2 } Scope This module will generate documentation from all Zig code that resides in the same code directory as the base module (or overridden directory, if applicable). Zig code in subdirectories will not be subjected to document generation."},{"ref":"Zigler.Unit.html","title":"Zigler.Unit","type":"module","doc":"Hooks your zig code into ExUnit, by converting zig tests into ExUnit tests. Usage Example Inside your zig code (dependent.zig): const beam = @import(&quot;beam.zig&quot;); const assert = beam.assert; fn one() i64 { return 1; } test &quot;the one function returns one&quot; { assert(one() == 1); } Inside your elixir code: defmodule MyZigModule do use Zigler ~Z&quot;&quot;&quot; const dependent = @import(&quot;dependent.zig&quot;); /// nif: some_nif_fn/1 ... &quot;&quot;&quot; end Inside your test module: defmodule MyZigTest do use ExUnit.Case, async: true use Zigler.Unit zigtest MyZigModule end Scope zigtest will run tests from the following sources: any tests inside of a sigil Z or sigil z construction any tests inside pub @import zig sources. any tests inside pub usingnamespace zig sources. recursively discovered pub structs."},{"ref":"Zigler.Unit.html#zigtest/2","title":"Zigler.Unit.zigtest/2","type":"macro","doc":"loads a module that wraps a zigler NIF, consults the corresponding zig code, generating the corresponding zig tests. Must be called from within a module that has use ExUnit.Case and use Zigler"},{"ref":"Zigler.Assembler.html","title":"Zigler.Assembler","type":"module","doc":"Fist phase of the Zigler compilation process. Looks through the contents of the zig code and creates a map of required source code files. Then outputs a struct containing the relevant information."},{"ref":"Zigler.Assembler.html#assemble_asset!/2","title":"Zigler.Assembler.assemble_asset!/2","type":"function","doc":""},{"ref":"Zigler.Assembler.html#assemble_assets!/2","title":"Zigler.Assembler.assemble_assets!/2","type":"function","doc":"assembles zig assets, taking them from their source to and putting them into the target directory."},{"ref":"Zigler.Assembler.html#assemble_kernel!/1","title":"Zigler.Assembler.assemble_kernel!/1","type":"function","doc":"assembles the &quot;core&quot; part of the zig adapter code, these are parts for shimming Zig libraries into the BEAM that are going to be @import&#39;d into the runtime by default."},{"ref":"Zigler.Assembler.html#parse_code/2","title":"Zigler.Assembler.parse_code/2","type":"function","doc":""},{"ref":"Zigler.Assembler.html#parse_file/2","title":"Zigler.Assembler.parse_file/2","type":"function","doc":""},{"ref":"Zigler.Assembler.html#t:file_type/0","title":"Zigler.Assembler.file_type/0","type":"type","doc":""},{"ref":"Zigler.Assembler.html#t:t/0","title":"Zigler.Assembler.t/0","type":"type","doc":""},{"ref":"Zigler.Code.html","title":"Zigler.Code","type":"module","doc":"all code responsible for generating zig code lives in this module."},{"ref":"Zigler.Code.html#adapter/1","title":"Zigler.Code.adapter/1","type":"function","doc":""},{"ref":"Zigler.Code.html#aggregate_includes/1","title":"Zigler.Code.aggregate_includes/1","type":"function","doc":""},{"ref":"Zigler.Code.html#c_imports/1","title":"Zigler.Code.c_imports/1","type":"function","doc":""},{"ref":"Zigler.Code.html#footer/1","title":"Zigler.Code.footer/1","type":"function","doc":""},{"ref":"Zigler.Code.html#generate_main/1","title":"Zigler.Code.generate_main/1","type":"function","doc":""},{"ref":"Zigler.Code.html#zig_imports/1","title":"Zigler.Code.zig_imports/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html","title":"Zigler.Code.LongRunning","type":"module","doc":"Generates code for long-running nifs. long-running functions require several parts to get right. a zig struct needs to be that holds arguments relvant to the long-running nif, these arguments need to be cleaned up in a sane fashion once the nif has completed running. This struct is going to be argetric on the nif arguments, and will be packed into a BEAM resource. a packer function which takes the beam arguments and shoves them into a zig struct to be stored in a BEAM resource, then launches the function, returning the resource. a launcher function which runs the packer wrapping the errors from the launch function. The launch function must be a nif function, as it will be called from the BEAM. a harness function which is passed the resource struct, and is responsible for updating the resource, sending back a &quot;finished&quot; message to parent context. The harness function will be responsible for wrapping the declared nif function. a fetching function which is passed the resource struct, and marshals it back into BEAM terms, returning the result. This function must be a nif function, as it will be called from the BEAM. a cleanup function which is responsible for cleaning up the resource object once the thread has completed its task."},{"ref":"Zigler.Code.LongRunning.html#adapter/1","title":"Zigler.Code.LongRunning.adapter/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#cache/1","title":"Zigler.Code.LongRunning.cache/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#cache_cleanup/1","title":"Zigler.Code.LongRunning.cache_cleanup/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#cache_ptr/1","title":"Zigler.Code.LongRunning.cache_ptr/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#cache_struct/1","title":"Zigler.Code.LongRunning.cache_struct/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#fetcher/1","title":"Zigler.Code.LongRunning.fetcher/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#fetcher_fn/1","title":"Zigler.Code.LongRunning.fetcher_fn/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#function_skeleton/1","title":"Zigler.Code.LongRunning.function_skeleton/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#harness/1","title":"Zigler.Code.LongRunning.harness/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#harness_fn/1","title":"Zigler.Code.LongRunning.harness_fn/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#launcher/1","title":"Zigler.Code.LongRunning.launcher/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#launcher_fn/1","title":"Zigler.Code.LongRunning.launcher_fn/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#packer/1","title":"Zigler.Code.LongRunning.packer/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#packer_fn/1","title":"Zigler.Code.LongRunning.packer_fn/1","type":"function","doc":""},{"ref":"Zigler.Compiler.html","title":"Zigler.Compiler","type":"module","doc":"handles instrumenting elixir code with hooks for zig NIFs."},{"ref":"Zigler.Compiler.html#assembly_dir/2","title":"Zigler.Compiler.assembly_dir/2","type":"function","doc":""},{"ref":"Zigler.Compiler.html#function_skeleton/1","title":"Zigler.Compiler.function_skeleton/1","type":"function","doc":""},{"ref":"Zigler.Compiler.html#precompile/1","title":"Zigler.Compiler.precompile/1","type":"function","doc":""},{"ref":"Zigler.Compiler.html#t:t/0","title":"Zigler.Compiler.t/0","type":"type","doc":""},{"ref":"Zigler.Doc.Parser.html","title":"Zigler.Doc.Parser","type":"module","doc":"handles parsing zig code for the purposes of documentation"},{"ref":"Zigler.Doc.Parser.html#docs_from_dir/2","title":"Zigler.Doc.Parser.docs_from_dir/2","type":"function","doc":""},{"ref":"Zigler.Doc.Parser.html#file_parser/2","title":"Zigler.Doc.Parser.file_parser/2","type":"function","doc":"Parses the given binary as file_parser. Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the file_parser (start position) as {line, column_on_line}. Options :line - the initial line, defaults to 1 :byte_offset - the initial byte offset, defaults to 0 :context - the initial context value. It will be converted to a map"},{"ref":"Zigler.Doc.Parser.html#head_parser/2","title":"Zigler.Doc.Parser.head_parser/2","type":"function","doc":"Parses the given binary as head_parser. Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the head_parser (start position) as {line, column_on_line}. Options :line - the initial line, defaults to 1 :byte_offset - the initial byte offset, defaults to 0 :context - the initial context value. It will be converted to a map"},{"ref":"Zigler.Doc.Parser.html#moduledocs_from_file/3","title":"Zigler.Doc.Parser.moduledocs_from_file/3","type":"function","doc":""},{"ref":"Zigler.Doc.Retriever.html","title":"Zigler.Doc.Retriever","type":"module","doc":"decides what features should be documented on zig doc. may be in flux."},{"ref":"Zigler.Doc.Retriever.html#docs_from_dir/2","title":"Zigler.Doc.Retriever.docs_from_dir/2","type":"function","doc":""},{"ref":"Zigler.Module.html","title":"Zigler.Module","type":"module","doc":"this struct represents all information a zigler module bound to a nif should have. WIP."},{"ref":"Zigler.Module.html#imports/1","title":"Zigler.Module.imports/1","type":"function","doc":""},{"ref":"Zigler.Module.html#t:t/0","title":"Zigler.Module.t/0","type":"type","doc":""},{"ref":"Zigler.Parser.html","title":"Zigler.Parser","type":"module","doc":"main parsing module: Handles general parsing of nif, resource directives and code elements succeeding those."},{"ref":"Zigler.Parser.html#parse/4","title":"Zigler.Parser.parse/4","type":"function","doc":""},{"ref":"Zigler.Parser.html#parse_zig_block/2","title":"Zigler.Parser.parse_zig_block/2","type":"function","doc":"Parses the given binary as parse_zig_block. Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the parse_zig_block (start position) as {line, column_on_line}. Options :line - the initial line, defaults to 1 :byte_offset - the initial byte offset, defaults to 0 :context - the initial context value. It will be converted to a map"},{"ref":"Zigler.Parser.html#t:line_info/0","title":"Zigler.Parser.line_info/0","type":"type","doc":""},{"ref":"Zigler.Parser.html#t:parsec_retval/0","title":"Zigler.Parser.parsec_retval/0","type":"type","doc":""},{"ref":"Zigler.Parser.html#t:t/0","title":"Zigler.Parser.t/0","type":"type","doc":""},{"ref":"Zigler.Parser.Error.html","title":"Zigler.Parser.Error","type":"module","doc":"parses errors emitted by the zig compiler"},{"ref":"Zigler.Parser.Error.html#backreference/2","title":"Zigler.Parser.Error.backreference/2","type":"function","doc":"given a code file path and a line number, calculates the file and line number of the source document from which it came. Strongly depends on having fencing comments of the form // ref: &lt;file&gt; line: &lt;line&gt; in order to backtrack this information."},{"ref":"Zigler.Parser.Error.html#check_ref/2","title":"Zigler.Parser.Error.check_ref/2","type":"function","doc":"Parses the given binary as check_ref. Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the check_ref (start position) as {line, column_on_line}. Options :line - the initial line, defaults to 1 :byte_offset - the initial byte offset, defaults to 0 :context - the initial context value. It will be converted to a map"},{"ref":"Zigler.Parser.Error.html#parse/2","title":"Zigler.Parser.Error.parse/2","type":"function","doc":"given a zig compiler error message, a directory for the code file, and the temporary directory where code assembly is taking place, return an appropriate CompileError struct which can be raised to emit a sensible compiler error message. The temporary directory is stripped (when reasonable) in favor of a &quot;true&quot; source file and any filename substitutions are performed as well."},{"ref":"Zigler.Parser.Error.html#parse_error/2","title":"Zigler.Parser.Error.parse_error/2","type":"function","doc":"Parses the given binary as parse_error. Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the parse_error (start position) as {line, column_on_line}. Options :line - the initial line, defaults to 1 :byte_offset - the initial byte offset, defaults to 0 :context - the initial context value. It will be converted to a map"},{"ref":"Zigler.Parser.Imports.html","title":"Zigler.Parser.Imports","type":"module","doc":"For parsing, looking for imports, cimports, and usingnamespace directives. To be completed later"},{"ref":"Zigler.Parser.Imports.html#parse/1","title":"Zigler.Parser.Imports.parse/1","type":"function","doc":""},{"ref":"Zigler.Parser.Imports.html#parse_imports/2","title":"Zigler.Parser.Imports.parse_imports/2","type":"function","doc":"Parses the given binary as parse_imports. Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the parse_imports (start position) as {line, column_on_line}. Options :line - the initial line, defaults to 1 :byte_offset - the initial byte offset, defaults to 0 :context - the initial context value. It will be converted to a map"},{"ref":"Zigler.Parser.Imports.html#t:line_info/0","title":"Zigler.Parser.Imports.line_info/0","type":"type","doc":""},{"ref":"Zigler.Parser.Imports.html#t:parsec_retval/0","title":"Zigler.Parser.Imports.parsec_retval/0","type":"type","doc":""},{"ref":"Zigler.Parser.Imports.html#t:t/0","title":"Zigler.Parser.Imports.t/0","type":"type","doc":""},{"ref":"Zigler.Parser.Nif.html","title":"Zigler.Parser.Nif","type":"module","doc":"This datastructure represents structured information about a single nif inside of a Zigler.sigil_Z/2 block. This is used to generate the exported_nifs variable which is an array of ErlNifFunc structs. The following keys are implemented: name: (atom/0) the function name to be bound into the module arity: (arity/0) the arity of the erlang function (the zig function may have a different arity). doc: (iodata/0) zig docstrings which should be turned into elixir docs args: (String.t/0) a list of zig types which are the arguments for the function retval: (String.t/0) the type of the return value opts: (t:keyword) list of nif options. long: true -- if the nif should run in a separate OS thread. dirty: :cpu -- if the nif should run in a dirty cpu scheduler. dirty: :io -- if the nif should run in a dirty io scheduler."},{"ref":"Zigler.Parser.Nif.html#register_function_header/2","title":"Zigler.Parser.Nif.register_function_header/2","type":"function","doc":""},{"ref":"Zigler.Parser.Nif.html#validate_args/3","title":"Zigler.Parser.Nif.validate_args/3","type":"function","doc":""},{"ref":"Zigler.Parser.Nif.html#validate_arity/3","title":"Zigler.Parser.Nif.validate_arity/3","type":"function","doc":""},{"ref":"Zigler.Parser.Nif.html#validate_retval/3","title":"Zigler.Parser.Nif.validate_retval/3","type":"function","doc":""},{"ref":"Zigler.Parser.Nif.html#t:option/0","title":"Zigler.Parser.Nif.option/0","type":"type","doc":""},{"ref":"Zigler.Parser.Nif.html#t:t/0","title":"Zigler.Parser.Nif.t/0","type":"type","doc":""},{"ref":"Zigler.Parser.Resource.html","title":"Zigler.Parser.Resource","type":"module","doc":"resource parser object"},{"ref":"Zigler.Parser.Resource.html#register_resource_definition/1","title":"Zigler.Parser.Resource.register_resource_definition/1","type":"function","doc":""},{"ref":"Zigler.Parser.ResourceCleanup.html","title":"Zigler.Parser.ResourceCleanup","type":"module","doc":"resource cleanup struct"},{"ref":"Zigler.Parser.ResourceCleanup.html#register_function_header/2","title":"Zigler.Parser.ResourceCleanup.register_function_header/2","type":"function","doc":""},{"ref":"Zigler.Parser.ResourceCleanup.html#validate_args/3","title":"Zigler.Parser.ResourceCleanup.validate_args/3","type":"function","doc":""},{"ref":"Zigler.Parser.ResourceCleanup.html#validate_arity/3","title":"Zigler.Parser.ResourceCleanup.validate_arity/3","type":"function","doc":""},{"ref":"Zigler.Parser.ResourceCleanup.html#validate_retval/3","title":"Zigler.Parser.ResourceCleanup.validate_retval/3","type":"function","doc":""},{"ref":"Zigler.Parser.Unit.html","title":"Zigler.Parser.Unit","type":"module","doc":"parses zig code and converts test blocks to test functions"},{"ref":"Zigler.Parser.Unit.html#parse/2","title":"Zigler.Parser.Unit.parse/2","type":"function","doc":""},{"ref":"Zigler.Parser.Unit.html#unit_parser/2","title":"Zigler.Parser.Unit.unit_parser/2","type":"function","doc":"Parses the given binary as unit_parser. Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the unit_parser (start position) as {line, column_on_line}. Options :line - the initial line, defaults to 1 :byte_offset - the initial byte offset, defaults to 0 :context - the initial context value. It will be converted to a map"},{"ref":"Zigler.Patches.html","title":"Zigler.Patches","type":"module","doc":"occasionally zig and/or the llvm bits that get pulled down have errors. This module actively patches code when this situation occurs."},{"ref":"Zigler.Patches.html#patch_needed?/2","title":"Zigler.Patches.patch_needed?/2","type":"function","doc":""},{"ref":"Zigler.Patches.html#sync/1","title":"Zigler.Patches.sync/1","type":"function","doc":""},{"ref":"Zigler.Typespec.html","title":"Zigler.Typespec","type":"module","doc":"handles creating typespecs for zig Nif functions"},{"ref":"Zigler.Zig.html","title":"Zigler.Zig","type":"module","doc":"contains all parts of the Zigler library involved in calling the zig compiler toolchain"},{"ref":"Zigler.Zig.html#compile/2","title":"Zigler.Zig.compile/2","type":"function","doc":""},{"ref":"Zigler.Zig.html#download_zig_archive/2","title":"Zigler.Zig.download_zig_archive/2","type":"function","doc":""},{"ref":"Zigler.Zig.html#fetch/1","title":"Zigler.Zig.fetch/1","type":"function","doc":""},{"ref":"Zigler.Zig.html#get_arch/0","title":"Zigler.Zig.get_arch/0","type":"function","doc":""},{"ref":"Zigler.Zig.html#get_os/0","title":"Zigler.Zig.get_os/0","type":"function","doc":""},{"ref":"Zigler.Zig.html#include_directories/1","title":"Zigler.Zig.include_directories/1","type":"function","doc":""},{"ref":"Zigler.Zig.html#unarchive_zig/1","title":"Zigler.Zig.unarchive_zig/1","type":"function","doc":""},{"ref":"beam.html","title":"beam","type":"module","doc":"This struct contains adapters designed to facilitate interfacing the BEAM&#39;s c-style helpers for NIFs with a more idiomatic Zig-style of programming, for example, the use of slices instead of null-terminated arrays as strings. This struct derives from zig/beam/beam.zig, and you may import it into your module&#39;s zig code by calling: const beam = @import(&quot;beam.zig&quot;) This is done automatically for you inside your ~Z forms, so do NOT use this import statement with inline Zig. Features The BEAM Allocator Wraps e.enif_alloc and e.enif_free functions into a compliant Zig allocator struct. You should thus be able to supply Zig standard library functions which require an allocator a struct that is compliant with its requirements. This is, in particular, useful for slice generation. Example (slice generation) beam = @import(&quot;beam.zig&quot;); fn make_a_slice_of_floats() ![]f32 { return beam.allocator.alloc(f32, 100); } Beacuse Zig features composable allocators, you can very easily implement custom allocators on top of the existing BEAM allocator. Getters Erlang&#39;s NIF interface provides a comprehensive set of methods to retrieve data out of BEAM terms. However, this set of methods presents an error handling scheme that is designed for C and inconsistent with the idiomatic scheme used for Zig best practices. A series of get functions is provided, implementing these methods in accordance to best practices. These include get/3, which is the generic method for getting scalar values, get_X, which are typed methods for retrieving scalar values, and get_slice_of/3, which is the generic method for retrieving a Zig slice from a BEAM list. Naturally, for all of these functions, you will have to provide the BEAM environment value. Examples const beam = @import(&quot;beam.zig&quot;); fn double_value(env: beam.env, value: beam.term) !f64 { return (try beam.get_f64(env, value)) * 2; } fn sum_float_list(env: beam.env, list: beam.term) !f64 { zig_list: []f64 = try beam.get_slice_of(f64, env, list); defer beam.allocator.free(zig_list); // don&#39;t forget to clean up! result: f64 = 0; for (list) |item| { result += item; } return result; } Makers A series of &quot;make&quot; functions is provided which allow for easy export of Zig values back to the BEAM. Typically, these functions are used in the automatic type marshalling performed by Zigler, however, you may want to be able to use them yourself to assemble BEAM datatypes not directly supported by Zig. For example, a custom tuple value. Example const beam = @import(&quot;beam.zig&quot;); const ok_slice=&quot;ok&quot;[0..]; fn to_ok_tuple(env: beam.env, value: i64) !beam.term { var tuple_slice: []term = try beam.allocator.alloc(beam.term, 2); defer beam.allocator.free(tuple_slice); tuple_slice[0] = beam.make_atom(env, ok_slice); tuple_slice[1] = beam.make_i64(env, value); return beam.make_tuple(env, tuple_slice); }"},{"ref":"beam.html#get/3","title":"beam.get/3","type":"function","doc":"A helper for marshalling values from the BEAM runtime into Zig. Use this function if you need support for Zig generics. Used internally to typcheck values coming into Zig slice. supported types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#get_c_int/2","title":"beam.get_c_int/2","type":"function","doc":"Takes a BEAM int term and returns a c_int value. Should only be used for C interop with Zig functions. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_c_uint/2","title":"beam.get_c_uint/2","type":"function","doc":"Takes a BEAM int term and returns a c_uint value. Should only be used for C interop with Zig functions. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_c_long/2","title":"beam.get_c_long/2","type":"function","doc":"Takes a BEAM int term and returns a c_long value. Should only be used for C interop with Zig functions. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_c_ulong/2","title":"beam.get_c_ulong/2","type":"function","doc":"Takes a BEAM int term and returns a c_ulong value. Should only be used for C interop with Zig functions. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_isize/2","title":"beam.get_isize/2","type":"function","doc":"Takes a BEAM int term and returns a isize value. Should only be used for C interop. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_usize/2","title":"beam.get_usize/2","type":"function","doc":"Takes a BEAM int term and returns a usize value. Zig idiomatically uses usize for its size values, so typically you should be using this function. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_u8/2","title":"beam.get_u8/2","type":"function","doc":"Takes a BEAM int term and returns a u8 value. Note that this conversion function checks to make sure it&#39;s in range (0..255). Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_u16/2","title":"beam.get_u16/2","type":"function","doc":"Takes a BEAM int term and returns a u16 value. Note that this conversion function checks to make sure it&#39;s in range (0..65535). Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_u32/2","title":"beam.get_u32/2","type":"function","doc":"Takes a BEAM int term and returns a u32 value. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_u64/2","title":"beam.get_u64/2","type":"function","doc":"Takes a BEAM int term and returns a u64 value. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_i32/2","title":"beam.get_i32/2","type":"function","doc":"Takes a BEAM int term and returns an i32 value. Note that this conversion function does not currently do range checking. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_i64/2","title":"beam.get_i64/2","type":"function","doc":"Takes a BEAM int term and returns an i64 value. Note that this conversion function does not currently do range checking. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_f16/2","title":"beam.get_f16/2","type":"function","doc":"Takes a BEAM float term and returns an f16 value. Note that this conversion function does not currently do range checking. Raises beam.Error.FunctionClauseError if the term is not float/0"},{"ref":"beam.html#get_f32/2","title":"beam.get_f32/2","type":"function","doc":"Takes a BEAM float term and returns an f32 value. Note that this conversion function does not currently do range checking. Raises beam.Error.FunctionClauseError if the term is not float/0"},{"ref":"beam.html#get_f64/2","title":"beam.get_f64/2","type":"function","doc":"Takes a BEAM float term and returns an f64 value. Raises beam.Error.FunctionClauseError if the term is not float/0"},{"ref":"beam.html#get_atom_slice/2","title":"beam.get_atom_slice/2","type":"function","doc":"Takes a BEAM atom term and retrieves it as a slice []u8 value. it&#39;s the caller&#39;s responsibility to make sure that the value is freed. Uses the standard beam.allocator allocator. If you require a custom allocator, use get_atom_slice_alloc/3 Raises beam.Error.FunctionClauseError if the term is not atom/0"},{"ref":"beam.html#get_atom_slice_alloc/3","title":"beam.get_atom_slice_alloc/3","type":"function","doc":"Takes a BEAM atom term and retrieves it as a slice []u8 value, with any allocator. Raises beam.Error.FunctionClauseError if the term is not atom/0"},{"ref":"beam.html#get_c_string/2","title":"beam.get_c_string/2","type":"function","doc":"Takes an BEAM Kernel.binary/0 term and retrieves a pointer to the binary data as a Zig c-string ([*c]u8). No memory is allocated for this operation. Should only be used for c interop functions. Note: this function could have unexpected results if your BEAM binary contains any zero byte values. Always use get_char_slice/2 when C-interop is not necessary. Raises beam.Error.FunctionClauseError if the term is not Kernel.binary/0"},{"ref":"beam.html#get_char_slice/2","title":"beam.get_char_slice/2","type":"function","doc":"Takes an BEAM Kernel.binary/0 term and retrieves it as a Zig character slice ([]u8) No memory is allocated for this operation. Raises beam.Error.FunctionClauseError if the term is not Kernel.binary/0"},{"ref":"beam.html#get_binary/2","title":"beam.get_binary/2","type":"function","doc":"Takes an BEAM Kernel.binary/0 term and returns the corresponding binary struct. Raises beam.Error.FunctionClauseError if the term is not Kernel.binary/0"},{"ref":"beam.html#get_pid/2","title":"beam.get_pid/2","type":"function","doc":"Takes an BEAM Kernel.pid/0 term and returns the corresponding pid struct. Note that this is a fairly opaque struct and you&#39;re on your own as to what you can do with this (for now), except as a argument for the e.enif_send function. Raises beam.Error.FunctionClauseError if the term is not Kernel.pid/0"},{"ref":"beam.html#self/1","title":"beam.self/1","type":"function","doc":"shortcut for e.enif_self, marshalling into zig error style. returns the pid value if it&#39;s env is a process-bound environment, otherwise returns beam.Error.FunctionClauseError."},{"ref":"beam.html#get_tuple/2","title":"beam.get_tuple/2","type":"function","doc":"Takes an Beam tuple/0 term and returns it as a slice of term structs. Does not allocate memory for this operation. Raises beam.Error.FunctionClauseError if the term is not tuple/0"},{"ref":"beam.html#get_list_length/2","title":"beam.get_list_length/2","type":"function","doc":"Takes a BEAM list/0 term and returns its length. Raises beam.Error.FunctionClauseError if the term is not list/0"},{"ref":"beam.html#get_head_and_iter/2","title":"beam.get_head_and_iter/2","type":"function","doc":"Iterates over a BEAM list/0. In this function, the list value will be modified to the tl of the BEAM list, and the return value will be the BEAM term. Raises beam.Error.FunctionClauseError if the term is not list/0"},{"ref":"beam.html#get_slice_of/3","title":"beam.get_slice_of/3","type":"function","doc":"A generic function which lets you convert a BEAM list/0 of homogeous type into a Zig slice. The resulting slice will be allocated using the beam allocator, with ownership passed to the caller. If you need to use a different allocator, use get_slice_of_alloc/4 Raises beam.Error.FunctionClauseError if the term is not list/0. Also raises beam.Error.FunctionClauseError if any of the terms is incompatible with the internal type supported internal types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#get_slice_of_alloc/4","title":"beam.get_slice_of_alloc/4","type":"function","doc":"Converts an BEAM list/0 of homogenous type into a Zig slice, but using any allocator you wish. ownership is passed to the caller. Raises beam.Error.FunctionClauseError if the term is not list/0. Also raises beam.Error.FunctionClauseError if any of the terms is incompatible with the internal type. supported internal types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#get_bool/2","title":"beam.get_bool/2","type":"function","doc":"Converts an BEAM boolean/0 into a Zig bool. Raises beam.Error.FunctionClauseError if the term is not boolean/0. May potentially raise an out of memory error, as it must make an allocation to perform its conversion."},{"ref":"beam.html#make/3","title":"beam.make/3","type":"function","doc":"A helper for marshalling values from Zig back into the runtime. Use this function if you need support for Zig generics. supported types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#make_u8/2","title":"beam.make_u8/2","type":"function","doc":"converts a char (u8) value into a BEAM integer/0."},{"ref":"beam.html#make_u16/2","title":"beam.make_u16/2","type":"function","doc":"converts a unsigned (u16) value into a BEAM integer/0."},{"ref":"beam.html#make_u32/2","title":"beam.make_u32/2","type":"function","doc":"converts a unsigned (u32) value into a BEAM integer/0."},{"ref":"beam.html#make_u64/2","title":"beam.make_u64/2","type":"function","doc":"converts a unsigned (u64) value into a BEAM integer/0."},{"ref":"beam.html#make_c_int/2","title":"beam.make_c_int/2","type":"function","doc":"converts a c_int value into a BEAM integer/0."},{"ref":"beam.html#make_c_uint/2","title":"beam.make_c_uint/2","type":"function","doc":"converts a c_uint value into a BEAM integer/0."},{"ref":"beam.html#make_c_long/2","title":"beam.make_c_long/2","type":"function","doc":"converts a c_long value into a BEAM integer/0."},{"ref":"beam.html#make_c_ulong/2","title":"beam.make_c_ulong/2","type":"function","doc":"converts a c_ulong value into a BEAM integer/0."},{"ref":"beam.html#make_isize/2","title":"beam.make_isize/2","type":"function","doc":"converts an isize value into a BEAM integer/0."},{"ref":"beam.html#make_usize/2","title":"beam.make_usize/2","type":"function","doc":"converts a usize value into a BEAM integer/0."},{"ref":"beam.html#make_i32/2","title":"beam.make_i32/2","type":"function","doc":"converts an i32 value into a BEAM integer/0."},{"ref":"beam.html#make_i64/2","title":"beam.make_i64/2","type":"function","doc":"converts an i64 value into a BEAM integer/0."},{"ref":"beam.html#make_f16/2","title":"beam.make_f16/2","type":"function","doc":"converts an f16 value into a BEAM float/0."},{"ref":"beam.html#make_f32/2","title":"beam.make_f32/2","type":"function","doc":"converts an f32 value into a BEAM float/0."},{"ref":"beam.html#make_f64/2","title":"beam.make_f64/2","type":"function","doc":"converts an f64 value into a BEAM float/0."},{"ref":"beam.html#make_atom/2","title":"beam.make_atom/2","type":"function","doc":"converts a Zig char slice ([]u8) into a BEAM atom/0."},{"ref":"beam.html#make_slice/2","title":"beam.make_slice/2","type":"function","doc":"converts a Zig char slice ([]u8) into a BEAM binary/0. no memory allocation inside of Zig is performed and the BEAM environment is responsible for the resulting binary. You are responsible for managing the allocation of the slice."},{"ref":"beam.html#make_c_string/2","title":"beam.make_c_string/2","type":"function","doc":"converts an c string ([*c]u8) into a BEAM binary/0. Mostly used for c interop. no memory allocation inside of Zig is performed and the BEAM environment is responsible for the resulting binary. You are responsible for managing the allocation of the slice."},{"ref":"beam.html#make_tuple/2","title":"beam.make_tuple/2","type":"function","doc":"converts a slice of terms into a BEAM tuple/0."},{"ref":"beam.html#make_term_list/2","title":"beam.make_term_list/2","type":"function","doc":"converts a slice of terms into a BEAM list/0."},{"ref":"beam.html#make_charlist/2","title":"beam.make_charlist/2","type":"function","doc":"converts a Zig char slice ([]u8) into a BEAM charlist/0."},{"ref":"beam.html#make_cstring_charlist/2","title":"beam.make_cstring_charlist/2","type":"function","doc":"converts a c string ([*c]u8) into a BEAM charlist/0."},{"ref":"beam.html#make_list/3","title":"beam.make_list/3","type":"function","doc":"A helper to make BEAM lists out of slices of term. Use this function if you need a generic listbuilding function. uses the BEAM allocator internally. If you would like to use a custom allocator, (for example an arena allocator, if you have very long lists), use make_list_alloc/4 supported internal types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#make_list_alloc/4","title":"beam.make_list_alloc/4","type":"function","doc":"A helper to make a BEAM t:Kernel.list out of terms, with any allocator. Use this function if you need a generic listbuilding function. supported internal types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#make_c_int_list/2","title":"beam.make_c_int_list/2","type":"function","doc":"converts a c_int slice ([]c_int) into a BEAM list of integer/0."},{"ref":"beam.html#make_c_long_list/2","title":"beam.make_c_long_list/2","type":"function","doc":"converts a c_long slice ([]c_long) into a BEAM list of integer/0."},{"ref":"beam.html#make_i32_list/2","title":"beam.make_i32_list/2","type":"function","doc":"converts an i32 slice ([]i32) into a BEAM list of integer/0."},{"ref":"beam.html#make_i64_list/2","title":"beam.make_i64_list/2","type":"function","doc":"converts an i64 slice ([]i64) into a BEAM list of integer/0."},{"ref":"beam.html#make_f16_list/2","title":"beam.make_f16_list/2","type":"function","doc":"converts an f16 slice ([]f16) into a BEAM list of float/0."},{"ref":"beam.html#make_f32_list/2","title":"beam.make_f32_list/2","type":"function","doc":"converts an f32 slice ([]f32) into a BEAM list of float/0."},{"ref":"beam.html#make_f64_list/2","title":"beam.make_f64_list/2","type":"function","doc":"converts an f64 slice ([]f64) into a BEAM list of float/0."},{"ref":"beam.html#make_bool/2","title":"beam.make_bool/2","type":"function","doc":"converts a bool value into a Kernel.boolean/0 value."},{"ref":"beam.html#make_nil/1","title":"beam.make_nil/1","type":"function","doc":"creates a beam nil value."},{"ref":"beam.html#make_ok_tuple/3","title":"beam.make_ok_tuple/3","type":"function","doc":"A helper to make {:ok, term} terms from arbitrarily-typed values. supported types: c_int c_long isize usize u8 i32 i64 f16 f32 f64 Use make_ok_term/2 to make ok tuples from generic terms. Use make_ok_atom/2 to make ok tuples with atom terms from slices."},{"ref":"beam.html#make_ok_binary/2","title":"beam.make_ok_binary/2","type":"function","doc":"A helper to make {:ok, binary} terms from slices"},{"ref":"beam.html#make_ok_atom/2","title":"beam.make_ok_atom/2","type":"function","doc":"A helper to make {:ok, atom} terms from slices"},{"ref":"beam.html#make_ok_term/2","title":"beam.make_ok_term/2","type":"function","doc":"A helper to make {:ok, term} terms in general"},{"ref":"beam.html#make_error_tuple/3","title":"beam.make_error_tuple/3","type":"function","doc":"A helper to make {:error, term} terms from arbitrarily-typed values. supported types: c_int c_long isize usize u8 i32 i64 f16 f32 f64 Use make_error_term/2 to make error tuples from generic terms. Use make_error_atom/2 to make atom errors from slices."},{"ref":"beam.html#make_error_atom/2","title":"beam.make_error_atom/2","type":"function","doc":"A helper to make {:error, atom} terms from slices"},{"ref":"beam.html#make_error_binary/2","title":"beam.make_error_binary/2","type":"function","doc":"A helper to make {:error, binary} terms from slices"},{"ref":"beam.html#make_error_term/2","title":"beam.make_error_term/2","type":"function","doc":"A helper to make {:error, term} terms in general"},{"ref":"beam.html#make_ref/1","title":"beam.make_ref/1","type":"function","doc":"Encapsulates e.enif_make_ref and allows it to return a FunctionClauseError. Raises beam.Error.FunctionClauseError if the term is not Kernel.pid/0"},{"ref":"beam.html#raise_enomem/1","title":"beam.raise_enomem/1","type":"function","doc":"This function is used to communicate :enomem back to the BEAM as an exception. The BEAM is potentially OOM-safe, and Zig lets you leverage that. OOM errors from beam.allocator can be converted to a generic erlang term that represents an exception. Returning this from your NIF results in a BEAM throw event."},{"ref":"beam.html#raise_function_clause_error/1","title":"beam.raise_function_clause_error/1","type":"function","doc":"This function is used to communicate :function_clause back to the BEAM as an exception. By default Zigler will do argument input checking on value ingress from the dynamic BEAM runtime to the static Zig runtime. You can also use this function to communicate a similar error by returning the resulting term from your NIF."},{"ref":"beam.html#raise_assertion_error/1","title":"beam.raise_assertion_error/1","type":"function","doc":"This function is used to communicate :assertion_error back to the BEAM as an exception. Used when running Zigtests, when trapping beam.AssertionError.AssertionError."},{"ref":"beam.html#assert/3","title":"beam.assert/3","type":"function","doc":"A function used to return assertion errors to a zigtest. Zig&#39;s std.assert() will panic the Zig runtime and therefore the entire BEAM VM, making it incompatible with Elixir&#39;s Unit tests. As the VM is required for certain functionality (like e.enif_alloc), a BEAM-compatible assert is necessary. When building zigtests, assert(...) calls get lexically converted to try beam.assert(...) calls."},{"ref":"beam.AssertionError.AssertionError.html","title":"beam.AssertionError.AssertionError","type":"exception","doc":"Translates to ExUnit.AssertionError. Mostly used in Zig unit tests. All test clauses in the directories of your Zig-enabled modules are converted to Zig functions with the inferred type !void. The beam.assert/1 function can throw this error as its error type. Zigler converts assert statements in test blocks to try beam.assert(...);"},{"ref":"beam.Error.FunctionClauseError.html","title":"beam.Error.FunctionClauseError","type":"exception","doc":"Translates to Elixir FunctionClauseError. This is the default mechanism for reporting that a Zigler nif function has been incorrectly passed a value from the Elixir BEAM runtime. This is very important, as Zig is statically typed. support for users to be able to throw this value in their own Zig functions is forthcoming."},{"ref":"erl_nif.html","title":"erl_nif","type":"module","doc":"Provides erlang&#39;s NIF convenience functions to your Zig code. This struct derives from zig/beam/erl_nif.zig. Usually brought in as const e = @import(&quot;erl_nif.zig&quot;).c;, leading to the aliased shortcut e. for example, to call the enif_alloc function, you would use the following code: const e = @import(&quot;erl_nif.zig&quot;).c; pub fn give_me_ten_bytes() ?*u8 { return e.enif_alloc(10); } refer to the erlang documentation for available functions"},{"ref":"readme.html","title":"Zigler","type":"extras","doc":"Zigler Library test status: Dependent package test status:"},{"ref":"readme.html#installation","title":"Zigler - Installation","type":"extras","doc":"Zigler is available in Hex, and the package can be installed by adding zigler to your list of dependencies in mix.exs: def deps do [ {:zigler, &quot;~&gt; 0.3.0&quot;, runtime: false} ] end"},{"ref":"readme.html#documentation","title":"Zigler - Documentation","type":"extras","doc":"Docs can be found at https://hexdocs.pm/zigler."},{"ref":"readme.html#currently-supported-platforms","title":"Zigler - Currently supported platforms","type":"extras","doc":"Linux FreeBSD (tested, but not subjected to CI) MacOS (I believe it works but is still offically untested) Nerves cross-compilation is supported out of the box."},{"ref":"readme.html#zig-nifs-made-easy","title":"Zigler - Zig Nifs made easy","type":"extras","doc":"Wouldn&#39;t it be nice if you could make NIFs as easily as you can use the asm keyword in C? This is now possible, using the magic of Zig. defmodule ExampleZig do use Zigler ~Z&quot;&quot;&quot; /// nif: example_fun/2 fn example_fun(value1: f64, value2: f64) bool { return value1 &gt; value2; } &quot;&quot;&quot; end iex&gt; ExampleZig.example_fun(0.1, 0.4) false iex&gt; ExampleZig.example_fun(0.8, -0.8) true Zigler will do automatic type marshalling between Elixir code and Zig code. It will also convert trickier types into types you care about, for example: defmodule ZigCollections do use Zigler ~Z&quot;&quot;&quot; /// nif: string_count/1 fn string_count(string: []u8) i64 { return @intCast(i64, string.len); } /// nif: list_sum/1 fn list_sum(array: []f64) f64 { var sum: f64 = 0.0; for(array) | item | { sum += item; } return sum; } &quot;&quot;&quot; end iex&gt; ZigCollections.string_count(&quot;hello zig&quot;) 9 iex&gt; ZigCollections.list_sum([1.0, 2.0, 3.0]) 6.0 Memory allocation with zigler is easy! A standard BEAM allocator is provided for you, so any zig code you import will play nice with the BEAM. defmodule Allocations do use Zigler ~Z&quot;&quot;&quot; /// nif: double_atom/1 fn double_atom(env: beam.env, string: []u8) beam.atom { var double_string = beam.allocator.alloc(u8, string.len * 2) catch { return beam.raise_enomem(env); }; defer beam.allocator.free(double_string); for (string) | char, i | { double_string[i] = char; double_string[i + string.len] = char; } return beam.make_atom(env, double_string); } &quot;&quot;&quot; end iex&gt; Allocations.double_atom(&quot;foo&quot;) :foofoo It is a goal for Zigler to make using it to bind C libraries easier than using C to bind C libraries. Here is an example: defmodule BlasDynamic do use Zigler, libs: [&quot;/usr/lib/x86_64-linux-gnu/blas/libblas.so&quot;], include: [&quot;/usr/include/x86_64-linux-gnu&quot;] ~Z&quot;&quot;&quot; const blas = @cImport({ @cInclude(&quot;cblas.h&quot;); }); /// nif: blas_axpy/3 fn blas_axpy(env: beam.env, a: f64, x: []f64, y: []f64) beam.term { if (x.len != y.len) { return beam.raise_function_clause_error(env); } blas.cblas_daxpy(@intCast(c_int, x.len), a, x.ptr, 1, y.ptr, 1); return beam.make_f64_list(env, y) catch { return beam.raise_function_clause_error(env); }; } &quot;&quot;&quot; end test &quot;we can use dynamically-linked blas&quot; do # returns aX+Y assert [11.0, 18.0] == BlasDynamic.blas_axpy(3.0, [2.0, 4.0], [5.0, 6.0]) end Zigler even has support for zig docstrings. defmodule AllTheDocs do use Zigler ~Z&quot;&quot;&quot; /// a zero-arity function which returns 47. /// nif: zero_arity/0 fn zeroarity() i64 { return 47; } &quot;&quot;&quot; end iex&gt; h AllTheDocs.zeroarity def zeroarity() a zero-arity function which returns 47."},{"ref":"readme.html#zigler-principles","title":"Zigler - Zigler Principles","type":"extras","doc":"Make doing the right thing easy. Use magic, but sparingly. Let the user see behind the curtain. Let the user dial in magic as they choose. Magic shouldn&#39;t get in the way"},{"ref":"nifs.html","title":"Using Nifs","type":"extras","doc":"Using Nifs Nifs are the entrypoint between your BEAM code and your C ABI code. Zigler provides semantics which are designed to make it easy to write safe"},{"ref":"nifs.html#basic-operation","title":"Using Nifs - Basic operation","type":"extras","doc":"/// nif: my_nif/1 fn my_nif(input: i32) i32 { return input + 1; } simply define a zig fn with desired input parameters. Zigler will generate a corresponding function in the surrounding Elixir module, and a mismatched value will raise a FunctionClauseError. The following types are accepted as inputs: bool u8 u16 i32 u32 i64 u64 c_int * c_uint * c_long * c_ulong * isize * usize beam.term beam.atom beam.pid *only use these types to interface with C code and libraries. Slices of all of the above, except u8, are also accepted (see below)."},{"ref":"nifs.html#example-with-slices","title":"Using Nifs - Example (with slices)","type":"extras","doc":"/// nif: sum/1 fn sum(input: []f32) f32 { var total: f32 = 0.0; for (input) | value | { total += value }; return total; }"},{"ref":"nifs.html#binaries","title":"Using Nifs - Binaries","type":"extras","doc":"Binaries are marshalled into u8 slices. /// nif: double/1 fn capitalize(input: []u8) []u8 { // note this is poorly bounds tested. input[0] = input[0] - 32 return input; }"},{"ref":"nifs.html#allocation","title":"Using Nifs - Allocation","type":"extras","doc":"Use the beam.allocator to allocate memory when you need it."},{"ref":"nifs.html#optional-beam-env-term","title":"Using Nifs - Optional beam.env term","type":"extras","doc":"In order to build terms to be consumed by the rest of the BEAM, you need to have access to the &quot;environment&quot; of the calling process. In order to be provided this, you may request it as the first parameter to your zig function. Play close attention to the arity of the nif declaration. /// nif: add_3/1 fn add_3(env: beam.env, number: i32) beam.term { return beam.make_i32(env, number + 3); }"},{"ref":"nifs.html#dirty-nifs","title":"Using Nifs - Dirty nifs","type":"extras","doc":"If you want to launch your nif as a dirty, use the dirty_cpu or dirty_io attributes. Note that by default, your vm will only have a limited number of dirty threads available and launching the nif may fail if all of them are occupied. /// nif: dirty_nif/1 dirty_cpu fn dirty_nif(env: beam.env, input: u64) u64 { ... // code that takes a long time ... return my_result; }"},{"ref":"nifs.html#future-feature-long","title":"Using Nifs - Future feature:  long","type":"extras","doc":"An upcoming feature will be the ability to run a function inside of a sidecar (OS) process. There is a considerable amount of boilerplate required to safely wrap launching such a process, and Zigler will take care of that for you. Example /// nif: my_nif/1 long fn my_nif(env: beam.env, input: beam.term) u64 { ... // code that takes a long time ... return my_result; } In the above example situation, the result my_result will be returned to the process which called it, which will block awaiting a completion message, without consuming NIF timeslices except for entry into the function (serialization of the input data). Optional parent pid example In some instances, you may want to monitor the parent pid from within the long-running NIF. In those situations, if you request the parent&#39;s pid as the second term you can for example, effectively monitor to silently quit out of loops in case the parent dies. If you don&#39;t want the parent pid to block awaiting a response, use the long:detached attribute. /// nif: my_nif/1 long:detached fn my_nif(env: beam.env, parent: beam.pid, input: beam.term) void { while (true) { ... // code inside of a loop ... if is_dead(env, parent) break; } } This attribute may be combined with the managed or dirty attributes. You should use dirty long nifs if you think serialization of your BEAM data structure will take a very long time."},{"ref":"nifs.html#future-feature-managed-and-managed-arena","title":"Using Nifs - Future feature:  managed and managed:arena","type":"extras","doc":"An upcoming feature will be the ability to use a &#39;managed&#39; allocation system. Within a managed nif, performing free operations on allocated data is optional, preventing the risk of memory leaks. All memory fetched from the managed allocator will be automatically freed on function exit. It is important to never pass these data into a resource. If you don&#39;t want to ever perform free operations you can use managed:arena which will use an arena allocation strategy, which can potentially offer better latency. Example /// nif: my_nif/1 managed fn my_nif(env: beam.env, input: beam.term) beam.term { beam.managed.alloc(u8, 100) ... } This attribute may be combined with the long attribute."},{"ref":"resources.html","title":"Using Resources","type":"extras","doc":"Using Resources Resources are C ABI objects which you declare should be managed by the BEAM reference-counted garbage collector. If you are passing data between function calls, you should pass the information using a reference. Note: this API is new and may be subject to change if it&#39;s found to be inefficient or difficult."},{"ref":"resources.html#basics-definition","title":"Using Resources - Basics: Definition","type":"extras","doc":"A resource is declared by using the /// resource: &lt;name&gt; definition directive. The name should match a zig type declared in the next line. Examples In the following example, the resource my_resource is bound to the builtin i64 type. This would allow you to store a mutable 64-bit integer and pass its pointer between function calls in the BEAM. NOTE: although it&#39;s mutable it is NOT threadsafe. /// resource: my_resource definition const my_resource = i64; In the next example, resource name_resource is bound to a custom struct type. This is a more typical situation, where structured data, possibly requiring internal memory allocation, needs to be passed between function calls. /// resource: name_resource definition const name_resource = struct { first_name: []u8, last_name: []u8, };"},{"ref":"resources.html#basics-creation","title":"Using Resources - Basics: Creation","type":"extras","doc":"In order to create a resource, a __resource__ struct is autogenerated which makes manipulating BEAM resources much simpler. When returned from the nif, the resource will appear as a reference/0 term. As this term is passed around, potentially between processes, it will only be garbage collected when all references have been to the term have been lost (more on that in cleanup). Note that you will only have access to the __resource__ struct in the context of code associated with your module in Zigler.sigil_Z/2 segments. Also note that resource references must remain local, the content cannot be automatically serialized and deserialized and transferred to another node. Example /// nif: create_name/2 fn create_name(env: beam.env, src_first:[]u8, src_last: []u8) beam.term { var first = beam.allocator.alloc(u8, src_first.len) catch return beam.raise_enomem(env); var last = beam.allocator.alloc(u8, src_last.len) catch { beam.allocator.free(first); return beam.raise_enomem(env); } std.mem.copy(u8, first, src_first); std.mem.copy(u8, last, src_last); return __resource__.create(name_resource, env, { .first_name = first; .last_name = last; }) catch beam.raise_function_clause_error(env); }"},{"ref":"resources.html#basics-fetching-and-updating","title":"Using Resources - Basics: Fetching and Updating","type":"extras","doc":"The __resource__ struct has update and fetch functions which allow you to retrieve or modify the contents in the struct. Note that although you can potentially access these data from different processes, it is not threadsafe by default without implementing mutexes around the processes. /// nif: update_last_name/2 fn update_last_name(env: beam.env, resource: beam.term, src_last: []u8) void { var new_last_name = beam.allocator.alloc(u8, src_last.len) catch return beam.raise_enomem(env); var full_name = __resource__.fetch(name_resource, env, value) catch return beam.raise_function_clause_error(env); // destroy the old last name and replace it with the new. beam.allocator.free(full_name.last_name); full_name.last_name = new_last_name; return __resource__.update(name_resource, env, resource, full_name) catch return beam.raise_function_clause_error(env); }"},{"ref":"resources.html#basics-cleanup","title":"Using Resources - Basics: Cleanup","type":"extras","doc":"Most resources should define a cleanup function. This function will be invoked when the last reference to that resource has been garbage collected by the virtual machine. The cleanup function is declared by using the /// resource: &lt;name&gt; definition, this should be followed by a void function taking a pointer to the resource. If there are any internal data that need freeing, you should do it here. Note: The memory for the resource itself will be cleaned up by Zigler, so don&#39;t free that. /// resource: name_resource cleanup fn name_resource_cleanup(env: beam.env, rsrc: *name_resource) void { beam.allocator.free(name_resource.first_name); beam.allocator.free(name_resource.last_name); # don&#39;t clean up the name_resource pointer }"}]